@@ -18,11 +18,11 @@ Struct::Path::PerlStyle - Perl-style syntax frontend for L<Struct::Path|Struct::
 
 =head1 VERSION
 
-Version 0.62
+Version 0.63
 
 =cut
 
-our $VERSION = '0.62';
+our $VERSION = '0.63';
 
 =head1 SYNOPSIS
 
@@ -224,15 +224,15 @@ sub ps_serialize($) {
                 croak "Incorrect array index '$i' (step #$sc)"
                     unless (looks_like_number($i) and int($i) == $i);
                 if (@ranges and (
-                    ($ranges[-1][1] + 1 == $i and $ranges[-1][0] <= $ranges[-1][1]) or   # ascending range
-                    ($ranges[-1][1] - 1 == $i and $ranges[-1][0] >= $ranges[-1][1])      # descending range
+                    ($ranges[-1][-1] + 1 == $i and $ranges[-1][0] < $i) or   # ascending
+                    ($ranges[-1][-1] - 1 == $i and $ranges[-1][0] > $i)      # descending
                 )) {
                     $ranges[-1][1] = $i; # update range
                 } else {
-                    push @ranges, [$i, $i];
+                    push @ranges, [$i]; # new range
                 }
             }
-            $out .= "[" . join(",", map { $_->[0] != $_->[1] ? "$_->[0]..$_->[1]" : $_->[0] } @{ranges}) . "]";
+            $out .= "[" . join(",", map { $_->[0] == $_->[-1] ? $_->[0] : "$_->[0]..$_->[-1]" } @{ranges}) . "]";
         } elsif (ref $step eq 'HASH') {
             my @items;
             if (keys %{$step} == 1 and exists $step->{keys} and ref $step->{keys} eq 'ARRAY' or not keys %{$step}) {
@@ -241,7 +241,7 @@ sub ps_serialize($) {
                         croak "Unsupported hash key type 'undef' (step #$sc)";
                     } elsif (ref $k) {
                         croak "Unsupported hash key type '" . (ref $k) . "' (step #$sc)";
-                    } elsif ($k =~ /^[0-9a-zA-Z_]+$/) {
+                    } elsif (looks_like_number($k) or $k =~ /^[0-9a-zA-Z_]+$/) {
                         # \w doesn't fit -- PPI can't parse unquoted utf8 hash keys
                         # https://github.com/adamkennedy/PPI/issues/168#issuecomment-180506979
                         push @items, $k;
